#!/usr/bin/env -S jq --slurp --raw-output -f

## Usage: mk-gap-stats.jq <json:gap-details> <json:stats>
##
## Generate data for all the analysis figures.
##
## Positional arguments:
##  <json:gap-details>   JSON output of `dentist check-results
##                       --gap-details=<json:gap-details>`
##  <json:stats>         JSON output of `dentist check-results --json`
##
## Examples:
##
## The average use case looks like this:
##
## $ dentist check --json --gap-details=gap-details.json \
## > true-assembly.dam test-reference.dam gap-closed.dam mapped > stats.json
## $ mk-gap-stats.jq gap-details.json stats.json > gap-stats.data

def bucketSize: 500;
def identityLevels: [
    1.0,
    0.999,
    0.99,
    0.95,
    0.90,
    0.70
];


def filter(f): map(select(f));


def correctnessDistribution:
    filter(.state == "closed") as $closedGaps |
    filter(.state == "broken") as $brokenGaps |
    filter(.state != "ignored") as $allGaps |
    identityLevels |
    map(
        . as $minIdentity |
        $closedGaps |
        filter($minIdentity <= .details.percentIdentity) |
        length
    ) + [
        ($closedGaps | length),
        ($brokenGaps | length),
        ($allGaps | length)
    ];

def weightedCorrectnessDistribution:
    filter(.state == "closed") as $closedGaps |
    filter(.state == "broken") as $brokenGaps |
    filter(.state != "ignored") as $allGaps |
    identityLevels |
    map(
        . as $minIdentity |
        $closedGaps |
        filter($minIdentity <= .details.percentIdentity) |
        map(.gapLength) | add // 0
    ) + [
        ($closedGaps | map(.gapLength) | add // 0),
        ($brokenGaps | map(.gapLength) | add // 0),
        ($allGaps | map(.gapLength) | add // 0)
    ];

def histogram:
    . as $gapDetails |
    (
        $gapDetails |
        filter(.state == "closed" or .state == "broken") |
        map(.gapLength) |
        max
    ) as $maxGapLength |
    if $maxGapLength then
        [range($maxGapLength / bucketSize | ceil)] |
        map(
            (. * bucketSize) as $minLength |
            ((. + 1) * bucketSize) as $supLength |
            $gapDetails |
            filter($minLength <= .gapLength and .gapLength < $supLength) |
            [$supLength] + correctnessDistribution
        )
    else
        []
    end;

def averageInsertionCorrectness:
    filter(.state == "closed") |
    {
        sum: (map(.details.percentIdentity // 0) | add),
        length: length,
        weightedSum: (map(.gapLength * (.details.percentIdentity // 0)) | add),
        totalWeight: (map(.gapLength) | add)
    } |
    if .length > 0 then
        [
            .sum / .length,
            .weightedSum / .totalWeight
        ]
    else
        [0, 0]
    end;

def prependHeader(headerNames):
    headerNames as $headerNames |
    [[
        ("# " + $headerNames[0]),
        $headerNames[1:][]
    ]] + .;

def namedDataset(name):
    "#\(name)", map(@tsv)[];


def identityLevelHeaders:
    identityLevels | map(
        if . == 1 then
            "%ID=\(.)"
        else
            "%ID>=\(.)"
        end
    );

def histogramDataset:
    .gapDetails |
    histogram |
    prependHeader([
        "gapLength<",
        identityLevelHeaders[],
        "closed",
        "broken",
        "existing"
    ]) |
    namedDataset("correctnessSizeDistribution");

def correctnessDistributionDataset:
    .gapDetails |
    [correctnessDistribution] |
    prependHeader([
        identityLevelHeaders[],
        "closed",
        "broken",
        "existing"
    ]) |
    namedDataset("correctnessDistribution");

def weightedCorrectnessDistributionDataset:
    .gapDetails |
    [weightedCorrectnessDistribution] |
    prependHeader([
        identityLevelHeaders[],
        "closed",
        "broken",
        "existing"
    ]) |
    namedDataset("weightedCorrectnessDistribution");

def averageInsertionCorrectnessDataset:
    .gapDetails |
    [averageInsertionCorrectness] |
    prependHeader([
        "averagePercentIdentity",
        "weightedAveragePercentIdentity"
    ]) |
    namedDataset("averageInsertionCorrectness");

def contigN50Dataset:
    .stats |
    [[.inputN50, .resultN50, .maximumN50]] |
    prependHeader([
        "input",
        "result",
        "maximum"
    ]) |
    namedDataset("contigN50");

# Main invocation
{
    stats: filter(has("inputN50"))[0],
    gapDetails: filter(has("state"))
} |
(
    histogramDataset,
    weightedCorrectnessDistributionDataset,
    correctnessDistributionDataset,
    averageInsertionCorrectnessDataset,
    contigN50Dataset
)
